unit StopTime;
{ $Revision: 2 $ }
{  $Archive: /Delphi 2.0/Lib/Stoptime.pas $ }
{ $Workfile: Stoptime.pas $ }
{   $Author: Cueb $ }
{     $Date: 23.03.97 16:17 $ }
{  $Modtime: 23.03.97 0:04 $ }
{  $History: Stoptime.pas $ }
{  }
{ *****************  Version 2  ***************** }
{ User: Cueb         Date: 23.03.97   Time: 16:17 }
{ Updated in $/Delphi 2.0/Lib/TimeUtils }
{ auf Millisekunden Genauigkeit umgestellt. }
{  }
{ *****************  Version 1  ***************** }
{ User: Cueb         Date: 22.03.97   Time: 23:31 }
{ Created in $/Delphi 2.0/Lib/TimeUtils }
{  $NoKeywords: $ }

interface

uses
	SysUtils,
	WinTypes,
	WinProcs,
	Messages,
	Classes,
	Graphics,
	Controls,
	Forms,
	Dialogs;

type
	TTimeAsStringEvent	= procedure (Hour, Min, Sec, MSec : Cardinal; var TimeString : String) of object;
	
	TStopTimer = class(TComponent)
	private
		{ Private-Deklarationen }
		FStartTime	: DWORD;
		FEndTime		: DWORD;
		FOnTimeAsString	: TTimeAsStringEvent;

		FHours			: Cardinal;
		FMins				: Cardinal;
		FSecs				: Cardinal;
		FMSecs			: Cardinal;
		FIterations	: Integer;

	protected
		{ Protected-Deklarationen }
		procedure SetIterations(AValue : Integer); virtual;
	public
		{ Public-Deklarationen }

		constructor Create(AOwner : TComponent); override;
		destructor Destroy; override;

		procedure	Stop; virtual;
		procedure	Start; virtual;
		procedure	Show; virtual;
		function	AsString : String;
	published
		{ Published-Deklarationen }

		property StartTime	: DWORD
			read	FStartTime;
		property EndTime	: DWORD
			read	FEndTime;
		property Hours	: Cardinal
			read	FHours;
		property Mins	: Cardinal
			read	FMins;
		property Secs	: Cardinal
			read	FSecs;
		property MSecs	: Cardinal
			read	FMSecs;
		property Iterations	: Integer
			read	FIterations
			write SetIterations
			default 1;
		property OnTimeAsString	: TTimeAsStringEvent
			read	FOnTimeAsString
			write FOnTimeAsString;
	end;

procedure Register;

implementation

uses
	MMSystem,
	LibConst;

constructor TStopTimer.Create(AOwner : TComponent);
begin
	inherited Create(AOwner);

	FIterations	:= 1;
end;

destructor TStopTimer.Destroy;
begin
	inherited;
end;

procedure	TStopTimer.Stop;
var
	ms	: DWORD;
begin
	FEndTime	:= TimeGetTime;
	ms	:= Round(FEndTime - FStartTime);

	FHours	:= ms div (60 * 60 * 1000);
	ms	:= ms - FHours * (60 * 60 * 1000);

	FMins	:= ms div (60 * 1000);
	ms	:= ms - FMins * (60 * 1000);

	FSecs		:= ms div 1000;
	FMSecs	:= ms - FSecs * 1000;
end;

procedure	TStopTimer.Start;
begin
	FStartTime	:= TimeGetTime;
	FHours			:= 0;
	FMins				:= 0;
	FSecs				:= 0;
	FMSecs			:= 0;
end;

procedure	TStopTimer.Show;
begin
	ShowMessage(AsString);
end;

function TStopTimer.AsString : String;
begin
	Result	:= format('%.3n', [FHours * 3600 + FMins * 60 + FSecs + FMSecs / 1000]);
	if Assigned(FOnTimeAsString) then begin
		FOnTimeAsString(FHours, FMins, FSecs, FMSecs, Result);
	end;
end;

procedure TStopTimer.SetIterations(AValue : Integer);
begin
	if AValue <> FIterations then begin
		FIterations	:= AValue;
	end;
	if FIterations < 1 then begin
		FIterations	:= 1;
	end;
end;

procedure Register;
begin
	RegisterComponents(LoadStr(srSystem), [
			TStopTimer]);
end;

end.
